@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2009 The Music Player Daemon Project
+ * (c) 2004-2010 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
  
  * This program is free software; you can redistribute it and/or modify
@@ -22,8 +22,9 @@
 
 #include "config.h"
 
+#include <glib.h>
+
 #include <stdbool.h>
-#include <stddef.h>
 
 #ifdef ENABLE_LOCALE
 const char *
@@ -33,10 +34,42 @@ charset_init(void);
 /**
  * Returns the number of terminal cells occupied by this string.
  */
+G_GNUC_PURE
 unsigned
 utf8_width(const char *str);
 
+/**
+ * Returns the number of terminal cells occupied by this string.
+ */
+G_GNUC_PURE
+unsigned
+locale_width(const char *p);
+
+/**
+ * Limits the width of the specified string.  Cuts it off before the
+ * specified width is exceeded.
+ *
+ * @return the resulting width of the string
+ */
+unsigned
+utf8_cut_width(char *p, unsigned max_width);
+
 char *utf8_to_locale(const char *str);
 char *locale_to_utf8(const char *str);
 
+/**
+ * Converts the UTF-8 string to the locale, and frees the source
+ * pointer.  Returns the source pointer as-is if no conversion is
+ * required.
+ */
+char *
+replace_utf8_to_locale(char *src);
+
+/**
+ * Converts the locale string to UTF-8, and frees the source pointer.
+ * Returns the source pointer as-is if no conversion is required.
+ */
+char *
+replace_locale_to_utf8(char *src);
+
 #endif
