@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2009 The Music Player Daemon Project
+ * (c) 2004-2010 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
  
  * This program is free software; you can redistribute it and/or modify
@@ -40,10 +40,13 @@ enum color {
 	COLOR_STATUS_BOLD,
 	COLOR_STATUS_TIME,
 	COLOR_STATUS_ALERT,
+	COLOR_DIRECTORY,
+	COLOR_PLAYLIST,
+	COLOR_BACKGROUND,
 	COLOR_END
 };
 
-short colors_str2color(const char *str);
+int colors_str2color(const char *str);
 
 #ifdef ENABLE_COLORS
 int colors_assign(const char *name, const char *value);
