@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2009 The Music Player Daemon Project
+ * (c) 2004-2010 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
  
  * This program is free software; you can redistribute it and/or modify
@@ -76,6 +76,8 @@
 #define CONF_WELCOME_SCREEN_LIST "welcome-screen-list"
 #define CONF_DISPLAY_TIME "display-time"
 #define CONF_JUMP_PREFIX_ONLY "jump-prefix-only"
+#define CONF_LYRICS_AUTOSAVE "lyrics-autosave"
+#define CONF_SECOND_COLUMN "second-column"
 
 static bool
 str2bool(char *str)
@@ -168,18 +170,20 @@ parse_key_definition(char *str)
 	return assign_keys(cmd, keys);
 }
 
-static const char *
+static bool
 parse_timedisplay_type(const char *str)
 {
-	if (!strcmp(str,"elapsed") || !strcmp(str,"remaining"))
-		return str;
+	if (strcmp(str, "elapsed") == 0)
+		return false;
+	else if (strcmp(str, "remaining") == 0)
+		return true;
 	else {
 		/* translators: ncmpc supports displaying the
 		   "elapsed" or "remaining" time of a song being
 		   played; in this case, the configuration file
 		   contained an invalid setting */
 		print_error(_("Bad time display type"), str);
-		return DEFAULT_TIMEDISPLAY_TYPE;
+		return false;
 	}
 }
 
@@ -298,10 +302,8 @@ check_screen_list(char *value)
 {
 	char **tmp = g_strsplit_set(value, " \t,", 100);
 	char **screen = NULL;
-	int i,j;
+	int i = 0, j = 0;
 
-	i=0;
-	j=0;
 	while( tmp && tmp[i] ) {
 		char *name = g_ascii_strdown(tmp[i], -1);
 		if (*name != '\0') {
@@ -369,13 +371,12 @@ get_search_mode(char *value)
 static bool
 parse_line(char *line)
 {
-	size_t len = strlen(line), i = 0, j;
+	size_t len = strlen(line), i = 0, j = 0;
 	char name[MAX_LINE_LENGTH];
 	char value[MAX_LINE_LENGTH];
 	bool match_found;
 
 	/* get the name part */
-	j = 0;
 	while (i < len && line[i] != '=' &&
 	       !g_ascii_isspace(line[i])) {
 		name[j++] = line[i++];
@@ -420,6 +421,8 @@ parse_line(char *line)
 	/* wide cursor */
 	else if (!strcasecmp(CONF_WIDE_CURSOR, name))
 		options.wide_cursor = str2bool(value);
+	else if (strcasecmp(name, "hardware-cursor") == 0)
+		options.hardware_cursor = str2bool(value);
 	/* welcome screen list */
 	else if (!strcasecmp(CONF_WELCOME_SCREEN_LIST, name))
 		options.welcome_screen_list = str2bool(value);
@@ -427,11 +430,10 @@ parse_line(char *line)
 	else if (!strcasecmp(CONF_VISIBLE_BITRATE, name))
 		options.visible_bitrate = str2bool(value);
 	/* timer display type */
-	else if (!strcasecmp(CONF_TIMEDISPLAY_TYPE, name)) {
-		g_free(options.timedisplay_type);
-		options.timedisplay_type=g_strdup(parse_timedisplay_type(value));
+	else if (!strcasecmp(CONF_TIMEDISPLAY_TYPE, name))
+		options.display_remaining_time = parse_timedisplay_type(value);
 		/* color definition */
-	} else if (!strcasecmp(CONF_COLOR_DEFINITION, name))
+	else if (!strcasecmp(CONF_COLOR_DEFINITION, name))
 #ifdef ENABLE_COLORS
 		parse_color_definition(value);
 #else
@@ -512,6 +514,18 @@ parse_line(char *line)
 		{}
 #else
 		options.jump_prefix_only = str2bool(value);
+#endif
+	else if (!strcasecmp(CONF_LYRICS_AUTOSAVE, name))
+#ifdef ENABLE_LYRICS_SCREEN
+		options.lyrics_autosave = str2bool(value);
+#else
+	{}
+#endif
+	else if (!strcasecmp(CONF_SECOND_COLUMN, name))
+#ifdef NCMPC_MINI
+		{}
+#else
+		options.second_column = str2bool(value);
 #endif
 	else
 		match_found = false;
