@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2009 The Music Player Daemon Project
+ * (c) 2004-2010 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
  
  * This program is free software; you can redistribute it and/or modify
@@ -27,10 +27,10 @@
 #define DEFAULT_SCREEN_LIST "playlist browse"
 
 /* song format - list window */
-#define DEFAULT_LIST_FORMAT "%name%|[%artist% - ]%title%|%shortfile%"
+#define DEFAULT_LIST_FORMAT "%name%|[%artist% - ][%title%|%shortfile%]"
 
 /* song format - status window */
-#define DEFAULT_STATUS_FORMAT "[%artist% - ]%title%|%shortfile%"
+#define DEFAULT_STATUS_FORMAT "[%artist% - ][%title%|%shortfile%]"
 
 #define DEFAULT_LYRICS_TIMEOUT 100
 
