@@ -1,17 +1,17 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2009 The Music Player Daemon Project
+ * (c) 2004-2010 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
- 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
@@ -20,14 +20,17 @@
 #include "config.h"
 #include "ncmpc.h"
 #include "mpdclient.h"
+#include "gidle.h"
 #include "charset.h"
 #include "options.h"
 #include "command.h"
 #include "ncu.h"
 #include "screen.h"
 #include "screen_utils.h"
+#include "screen_message.h"
 #include "strfsong.h"
 #include "i18n.h"
+#include "player_command.h"
 
 #ifndef NCMPC_MINI
 #include "conf.h"
@@ -41,6 +44,8 @@
 #include "lirc.h"
 #endif
 
+#include <mpd/client.h>
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <signal.h>
@@ -50,56 +55,18 @@
 #include <locale.h>
 #endif
 
-/* time between mpd updates [s] */
+/* time between mpd updates [ms] */
 static const guint update_interval = 500;
 
 #define BUFSIZE 1024
 
-static const guint idle_interval = 500;
-
-static mpdclient_t *mpd = NULL;
-static gboolean connected = FALSE;
+static struct mpdclient *mpd = NULL;
 static GMainLoop *main_loop;
-static guint reconnect_source_id, idle_source_id, update_source_id;
+static guint reconnect_source_id, update_source_id;
 
-static const gchar *
-error_msg(const gchar *msg)
-{
-	gchar *p;
-
-	if ((p = strchr(msg, '}')) == NULL)
-		return msg;
-
-	do {
-		p++;
-	} while (*p == '}' || * p== ' ');
-
-	return p;
-}
-
-static void
-error_callback(G_GNUC_UNUSED mpdclient_t *c, gint error, const gchar *_msg)
-{
-	char *msg = utf8_to_locale(_msg);
-
-	error = error & 0xFF;
-	switch (error) {
-	case MPD_ERROR_CONNPORT:
-	case MPD_ERROR_NORESPONSE:
-		break;
-	case MPD_ERROR_ACK:
-		screen_status_printf("%s", error_msg(msg));
-		screen_bell();
-		break;
-	default:
-		screen_status_printf("%s", msg);
-		screen_bell();
-		doupdate();
-		connected = FALSE;
-	}
-
-	g_free(msg);
-}
+#ifndef NCMPC_MINI
+static guint check_key_bindings_source_id;
+#endif
 
 #ifndef NCMPC_MINI
 static void
@@ -107,8 +74,8 @@ update_xterm_title(void)
 {
 	static char title[BUFSIZE];
 	char tmp[BUFSIZE];
-	mpd_Status *status = NULL;
-	mpd_Song *song = NULL;
+	struct mpd_status *status = NULL;
+	const struct mpd_song *song = NULL;
 
 	if (mpd) {
 		status = mpd->status;
@@ -116,7 +83,7 @@ update_xterm_title(void)
 	}
 
 	if (options.xterm_title_format && status && song &&
-	    IS_PLAYING(status->state))
+	    mpd_status_get_state(status) == MPD_STATE_PLAY)
 		strfsong(tmp, BUFSIZE, options.xterm_title_format, song);
 	else
 		g_strlcpy(tmp, PACKAGE " version " VERSION, BUFSIZE);
@@ -141,14 +108,6 @@ exit_and_cleanup(void)
 		mpdclient_disconnect(mpd);
 		mpdclient_free(mpd);
 	}
-
-	g_free(options.host);
-	g_free(options.password);
-	g_free(options.list_format);
-	g_free(options.status_format);
-#ifndef NCMPC_MINI
-	g_free(options.scroll_sep);
-#endif
 }
 
 static void
@@ -167,9 +126,9 @@ catch_sigcont(G_GNUC_UNUSED int sig)
 void
 sigstop(void)
 {
-  def_prog_mode();  /* save the tty modes */
-  endwin();         /* end curses mode temporarily */
-  kill(0, SIGSTOP); /* issue SIGSTOP */
+	def_prog_mode();  /* save the tty modes */
+	endwin();         /* end curses mode temporarily */
+	kill(0, SIGSTOP); /* issue SIGSTOP */
 }
 
 static guint timer_sigwinch_id;
@@ -197,9 +156,81 @@ catch_sigwinch(G_GNUC_UNUSED int sig)
 	timer_sigwinch_id = g_timeout_add(100, timer_sigwinch, NULL);
 }
 
+static void
+idle_callback(enum mpd_error error,
+	      G_GNUC_UNUSED enum mpd_server_error server_error,
+	      const char *message, enum mpd_idle events,
+	      G_GNUC_UNUSED void *ctx);
+
 static gboolean
 timer_mpd_update(gpointer data);
 
+static void
+enable_update_timer(void)
+{
+	if (update_source_id != 0)
+		return;
+
+	update_source_id = g_timeout_add(update_interval,
+					 timer_mpd_update, NULL);
+}
+
+static void
+disable_update_timer(void)
+{
+	if (update_source_id == 0)
+		return;
+
+	g_source_remove(update_source_id);
+	update_source_id = 0;
+}
+
+static bool
+should_enable_update_timer(void)
+{
+	return (mpdclient_is_connected(mpd) &&
+		(mpd->source == NULL || /* "idle" not supported */
+		 (mpd->status != NULL &&
+		  mpd_status_get_state(mpd->status) == MPD_STATE_PLAY)))
+#ifndef NCMPC_MINI
+		|| options.display_time
+#endif
+		;
+}
+
+static void
+auto_update_timer(void)
+{
+	if (should_enable_update_timer())
+		enable_update_timer();
+	else
+		disable_update_timer();
+}
+
+static void
+check_reconnect(void);
+
+static void
+do_mpd_update(void)
+{
+	if (mpdclient_is_connected(mpd) &&
+	    (mpd->source == NULL || mpd->events != 0 ||
+	     (mpd->status != NULL &&
+	      mpd_status_get_state(mpd->status) == MPD_STATE_PLAY)))
+		mpdclient_update(mpd);
+
+#ifndef NCMPC_MINI
+	if (options.enable_xterm_title)
+		update_xterm_title();
+#endif
+
+	screen_update(mpd);
+	mpd->events = 0;
+
+	mpdclient_put_connection(mpd);
+	check_reconnect();
+}
+
 /**
  * This timer is installed when the connection to the MPD server is
  * broken.  It tries to recover by reconnecting periodically.
@@ -207,64 +238,128 @@ timer_mpd_update(gpointer data);
 static gboolean
 timer_reconnect(G_GNUC_UNUSED gpointer data)
 {
-	int ret;
+	bool success;
+	struct mpd_connection *connection;
 
-	if (connected)
-		return FALSE;
+	assert(!mpdclient_is_connected(mpd));
+
+	reconnect_source_id = 0;
 
 	screen_status_printf(_("Connecting to %s...  [Press %s to abort]"),
 			     options.host, get_key_names(CMD_QUIT,0) );
 	doupdate();
 
 	mpdclient_disconnect(mpd);
-	ret = mpdclient_connect(mpd,
-				options.host, options.port,
-				1.5,
-				options.password);
-	if (ret != 0) {
+	success = mpdclient_connect(mpd,
+				    options.host, options.port,
+				    1.5,
+				    options.password);
+	if (!success) {
 		/* try again in 5 seconds */
-		g_timeout_add(5000, timer_reconnect, NULL);
+		reconnect_source_id = g_timeout_add(5000,
+						    timer_reconnect, NULL);
 		return FALSE;
 	}
 
+	connection = mpdclient_get_connection(mpd);
+
 #ifndef NCMPC_MINI
 	/* quit if mpd is pre 0.11.0 - song id not supported by mpd */
-	if (MPD_VERSION_LT(mpd, 0, 11, 0)) {
+	if (mpd_connection_cmp_server_version(connection, 0, 12, 0) < 0) {
+		const unsigned *version =
+			mpd_connection_get_server_version(connection);
 		screen_status_printf(_("Error: MPD version %d.%d.%d is to old (%s needed)"),
-				     mpd->connection->version[0],
-				     mpd->connection->version[1],
-				     mpd->connection->version[2],
-				     "0.11.0");
+				     version[0], version[1], version[2],
+				     "0.12.0");
 		mpdclient_disconnect(mpd);
 		doupdate();
 
 		/* try again after 30 seconds */
-		g_timeout_add(30000, timer_reconnect, NULL);
+		reconnect_source_id = g_timeout_add(30000,
+						    timer_reconnect, NULL);
 		return FALSE;
 	}
 #endif
 
-	screen_status_printf(_("Connected to %s"), options.host);
+	if (mpd_connection_cmp_server_version(connection,
+					      0, 14, 0) >= 0)
+		mpd->source = mpd_glib_new(connection,
+					   idle_callback, mpd);
+
+	screen_status_printf(_("Connected to %s"),
+			     options.host != NULL
+			     ? options.host : "localhost");
 	doupdate();
 
-	connected = TRUE;
-
 	/* update immediately */
-	g_timeout_add(1, timer_mpd_update, GINT_TO_POINTER(FALSE));
+	mpd->events = MPD_IDLE_DATABASE|MPD_IDLE_STORED_PLAYLIST|
+		MPD_IDLE_QUEUE|MPD_IDLE_PLAYER|MPD_IDLE_MIXER|MPD_IDLE_OUTPUT|
+		MPD_IDLE_OPTIONS|MPD_IDLE_UPDATE;
+
+	do_mpd_update();
+
+	auto_update_timer();
 
-	reconnect_source_id = 0;
 	return FALSE;
-
 }
 
-static gboolean
-timer_mpd_update(gpointer data)
+static void
+check_reconnect(void)
 {
-	if (connected)
-		mpdclient_update(mpd);
-	else if (reconnect_source_id == 0)
+	if (!mpdclient_is_connected(mpd) && reconnect_source_id == 0)
+		/* reconnect when the connection is lost */
 		reconnect_source_id = g_timeout_add(1000, timer_reconnect,
 						    NULL);
+}
+
+/**
+ * This function is called by the gidle.c library when MPD sends us an
+ * idle event (or when the connectiond dies).
+ */
+static void
+idle_callback(enum mpd_error error, enum mpd_server_error server_error,
+	      const char *message, enum mpd_idle events,
+	      void *ctx)
+{
+	struct mpdclient *c = ctx;
+	struct mpd_connection *connection;
+
+	c->idle = false;
+
+	connection = mpdclient_get_connection(c);
+	assert(connection != NULL);
+
+	if (error != MPD_ERROR_SUCCESS) {
+		char *allocated;
+
+		if (error == MPD_ERROR_SERVER &&
+		    server_error == MPD_SERVER_ERROR_UNKNOWN_CMD) {
+			/* the "idle" command is not supported - fall
+			   back to timer based polling */
+			mpd_glib_free(c->source);
+			c->source = NULL;
+			auto_update_timer();
+			return;
+		}
+
+		if (error == MPD_ERROR_SERVER)
+			message = allocated = utf8_to_locale(message);
+		else
+			allocated = NULL;
+		screen_status_message(message);
+		g_free(allocated);
+		screen_bell();
+		doupdate();
+
+		mpdclient_disconnect(c);
+		screen_update(mpd);
+		reconnect_source_id = g_timeout_add(1000, timer_reconnect,
+						    NULL);
+		return;
+	}
+
+	c->events |= events;
+	mpdclient_update(c);
 
 #ifndef NCMPC_MINI
 	if (options.enable_xterm_title)
@@ -272,32 +367,38 @@ timer_mpd_update(gpointer data)
 #endif
 
 	screen_update(mpd);
+	c->events = 0;
 
-	return GPOINTER_TO_INT(data);
+	mpdclient_put_connection(c);
+	check_reconnect();
+	auto_update_timer();
 }
 
-/**
- * This idle timer is invoked when the user hasn't typed a key for
- * 500ms.  It is used for delayed seeking.
- */
 static gboolean
-timer_idle(G_GNUC_UNUSED gpointer data)
+timer_mpd_update(G_GNUC_UNUSED gpointer data)
 {
-	screen_idle(mpd);
-	return TRUE;
+	do_mpd_update();
+
+	if (should_enable_update_timer())
+		return true;
+	else {
+		update_source_id = 0;
+		return false;
+	}
 }
 
 void begin_input_event(void)
 {
-	/* remove the idle timeout; add it later with fresh interval */
-	g_source_remove(idle_source_id);
 }
 
 void end_input_event(void)
 {
 	screen_update(mpd);
+	mpd->events = 0;
 
-	idle_source_id = g_timeout_add(idle_interval, timer_idle, NULL);
+	mpdclient_put_connection(mpd);
+	check_reconnect();
+	auto_update_timer();
 }
 
 int do_input_event(command_t cmd)
@@ -309,13 +410,9 @@ int do_input_event(command_t cmd)
 
 	screen_cmd(mpd, cmd);
 
-	if (cmd == CMD_VOLUME_UP || cmd == CMD_VOLUME_DOWN) {
+	if (cmd == CMD_VOLUME_UP || cmd == CMD_VOLUME_DOWN)
 		/* make sure we don't update the volume yet */
-		g_source_remove(update_source_id);
-		update_source_id = g_timeout_add(update_interval,
-						 timer_mpd_update,
-						 GINT_TO_POINTER(TRUE));
-	}
+		disable_update_timer();
 
 	return 0;
 }
@@ -352,10 +449,12 @@ timer_check_key_bindings(G_GNUC_UNUSED gpointer data)
 	gboolean key_error;
 
 	key_error = check_key_bindings(NULL, buf, sizeof(buf));
-	if (!key_error)
+	if (!key_error) {
 		/* no error: disable this timer for the rest of this
 		   process */
+		check_key_bindings_source_id = 0;
 		return FALSE;
+	}
 
 #ifdef ENABLE_KEYDEF_SCREEN
 	g_strchomp(buf);
@@ -486,7 +585,6 @@ main(int argc, const char *argv[])
 
 	/* create mpdclient instance */
 	mpd = mpdclient_new();
-	mpdclient_install_error_callback(mpd, error_callback);
 
 	/* initialize curses */
 	screen_init(mpd);
@@ -510,13 +608,11 @@ main(int argc, const char *argv[])
 	/* attempt to connect */
 	reconnect_source_id = g_timeout_add(1, timer_reconnect, NULL);
 
-	update_source_id = g_timeout_add(update_interval,
-					 timer_mpd_update,
-					 GINT_TO_POINTER(TRUE));
+	auto_update_timer();
+
 #ifndef NCMPC_MINI
-	g_timeout_add(10000, timer_check_key_bindings, NULL);
+	check_key_bindings_source_id = g_timeout_add(10000, timer_check_key_bindings, NULL);
 #endif
-	idle_source_id = g_timeout_add(idle_interval, timer_idle, NULL);
 
 	screen_paint(mpd);
 
@@ -524,6 +620,18 @@ main(int argc, const char *argv[])
 
 	/* cleanup */
 
+	cancel_seek_timer();
+
+	disable_update_timer();
+
+	if (reconnect_source_id != 0)
+		g_source_remove(reconnect_source_id);
+
+#ifndef NCMPC_MINI
+	if (check_key_bindings_source_id != 0)
+		g_source_remove(check_key_bindings_source_id);
+#endif
+
 	g_main_loop_unref(main_loop);
 	g_io_channel_unref(keyboard_channel);
 
@@ -540,6 +648,7 @@ main(int argc, const char *argv[])
 #endif
 
 	ncu_deinit();
+	options_deinit();
 
 	return 0;
 }
