@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2009 The Music Player Daemon Project
+ * (c) 2004-2010 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
  
  * This program is free software; you can redistribute it and/or modify
@@ -40,10 +40,14 @@ match_line(const char *line, const char *needle)
 	char *line_folded = locale_casefold(line);
 	char *needle_folded = locale_casefold(needle);
 
+#if GLIB_CHECK_VERSION(2,14,0)
 	bool ret = (bool)g_regex_match_simple((const gchar*)needle_folded,
 			(const gchar*)line_folded,
 			G_REGEX_CASELESS | G_REGEX_DOTALL | G_REGEX_OPTIMIZE,
 			0);
+#else
+	bool ret = strstr(line_folded, needle_folded) != NULL;
+#endif
 
 	g_free(line_folded);
 	g_free(needle_folded);
