@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2009 The Music Player Daemon Project
+ * (c) 2004-2010 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
  
  * This program is free software; you can redistribute it and/or modify
@@ -24,40 +24,19 @@
 #include "list_window.h"
 #include "command.h"
 
-#ifdef HAVE_NCURSESW_NCURSES_H
-#include <ncursesw/ncurses.h>
-#else
-#include <ncurses.h>
-#endif
-
 struct mpdclient;
 
 /* sound an audible and/or visible bell */
 void screen_bell(void);
 
 /* read a character from the status window */
-int screen_getch(WINDOW *w, const char *prompt);
+int screen_getch(const char *prompt);
 
-/* read a string from the status window */
-char *screen_getstr(WINDOW *w, const char *prompt);
-char *screen_readln(WINDOW *w, const char *prompt, const char *value,
+char *
+screen_read_password(const char *prompt);
+
+char *screen_readln(const char *prompt, const char *value,
 		    GList **history, GCompletion *gcmp);
-char *screen_readln_masked(WINDOW *w, const char *prompt);
-char *screen_read_pasword(WINDOW *w, const char *prompt);
-/* query user for a string and find it in a list window */
-int screen_find(struct list_window *lw,
-		int rows,
-		command_t findcmd,
-		list_window_callback_fn_t callback_fn,
-		void *callback_data);
-
-/* query user for a string and jump to the entry
- * which begins with this string while the users types */
-void screen_jump(struct list_window *lw,
-		list_window_callback_fn_t callback_fn,
-		void *callback_data);
-
-gint screen_auth(struct mpdclient *c);
 
 void screen_display_completion_list(GList *list);
 
