@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2009 The Music Player Daemon Project
+ * (c) 2004-2010 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
  
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,8 @@
 #define STRFSONG_H
 
 #include <glib.h>
-#include "libmpdclient.h"
+
+struct mpd_song;
 
 gsize strfsong(gchar *s, gsize max, const gchar *format,
 	       const struct mpd_song *song);
