@@ -1,5 +1,5 @@
 /* ncmpc (Ncurses MPD Client)
- * (c) 2004-2009 The Music Player Daemon Project
+ * (c) 2004-2010 The Music Player Daemon Project
  * Project homepage: http://musicpd.org
 
  * This program is free software; you can redistribute it and/or modify
@@ -93,8 +93,7 @@ byte_to_screen(const gchar *data, size_t x)
 
 	dup = g_strdup(data);
 	dup[x] = 0;
-	p = locale_to_utf8(dup);
-	g_free(dup);
+	p = replace_locale_to_utf8(dup);
 
 	width = utf8_width(p);
 	g_free(p);
